import time
from typing import Optional

from cpl_discord.service import DiscordBotServiceABC
from cpl_translation import TranslatePipe
from selenium.common import NoSuchElementException, StaleElementReferenceException
from selenium.webdriver import Keys
from selenium.webdriver.common.by import By
from selenium.webdriver.remote.webelement import WebElement
from selenium.webdriver.support import expected_conditions
from selenium.webdriver.support.wait import WebDriverWait

from ui_tests.src.ui_tests_shared.command_selectors_enum import CommandSelectorsEnum
from ui_tests_shared.test_case_with_app import TestCaseWithApp
from ui_tests_shared.ui import UI


class CommandTestCaseWithApp(TestCaseWithApp):
    _cmd: WebElement

    _bot: Optional[DiscordBotServiceABC] = None
    _t: Optional[TranslatePipe] = None

    @classmethod
    def setUpClass(cls):
        TestCaseWithApp.setUpClass()
        cls._bot = cls._services.get_service(DiscordBotServiceABC)
        cls._t = cls._services.get_service(TranslatePipe)

    @classmethod
    def send_message(cls, msg: str):
        if UI.driver.current_url != cls._test_settings.cmd_url:
            UI.driver.get(cls._test_settings.cmd_url)
            time.sleep(2)

        cmd_element_ident = (By.XPATH, CommandSelectorsEnum.cmd_chat.value)
        cmd_element = UI.driver.find_element(*cmd_element_ident)
        cmd_element.send_keys(msg)
        time.sleep(2)
        UI.driver.find_element(By.XPATH, CommandSelectorsEnum.msg_input.value).send_keys(Keys.ENTER)

    @classmethod
    def send_command(cls, cmd: str, selector: CommandSelectorsEnum):
        if UI.driver.current_url != cls._test_settings.cmd_url:
            UI.driver.get(cls._test_settings.cmd_url)
            time.sleep(2)

        cmd_element_ident = (By.XPATH, CommandSelectorsEnum.cmd_chat.value)
        cmd_element = UI.driver.find_element(*cmd_element_ident)
        cmd_element.send_keys(f'/{cmd}')
        time.sleep(2)

        ignored_exceptions = (NoSuchElementException, StaleElementReferenceException,)
        WebDriverWait(UI.driver, 20, ignored_exceptions=ignored_exceptions).until(
            expected_conditions.presence_of_element_located((
                By.XPATH,
                selector.value
            ))
        ).click()
        time.sleep(2)
        UI.driver.find_element(By.XPATH, CommandSelectorsEnum.cmd_chat_input.value).send_keys(Keys.ENTER)
