import time
import unittest
from typing import Optional

from cpl_core.configuration import ConfigurationABC
from cpl_core.dependency_injection import ServiceProviderABC
from selenium import webdriver
from selenium.webdriver import Keys
from selenium.webdriver.common.by import By
from selenium.webdriver.support import expected_conditions
from selenium.webdriver.support.wait import WebDriverWait

from ui_tests.main import get_app
from ui_tests_shared.configuration.test_settings import TestSettings
from ui_tests_shared.declarations import Declarations


class UI:
    _test_settings: Optional[TestSettings] = None

    options = webdriver.ChromeOptions()
    options.add_argument("user-data-dir=selenium-data")
    options.add_experimental_option('useAutomationExtension', False)
    options.add_experimental_option("excludeSwitches", ["enable-automation"])
    options.add_experimental_option("prefs", {
        "profile.default_content_setting_values.media_stream_mic": 1,
        "profile.default_content_setting_values.media_stream_camera": 1,
        "profile.default_content_setting_values.geolocation": 1,
        "profile.default_content_setting_values.notifications": 1
    })
    driver = webdriver.Chrome(options=options)

    _is_logged_in = False

    @classmethod
    def set_settings(cls, test_settings: TestSettings):
        cls._test_settings = test_settings

    @classmethod
    def login(cls, mail: str, password: str):
        if cls._is_logged_in:
            return

        # use full xpath: https://stackoverflow.com/questions/71179006/how-can-selenium-python-chrome-find-web-elements-visible-in-dev-tools-but-no

        cls.driver.get(cls._test_settings.login_url)

        try:
            WebDriverWait(cls.driver, 10).until(expected_conditions.url_matches(cls._test_settings.me_page_url))
            cls._is_logged_in = True
            return
        except Exception as e:
            WebDriverWait(cls.driver, 20).until(expected_conditions.presence_of_element_located((By.NAME, 'email')))

        mail_element = cls.driver.find_element(By.NAME, 'email')
        mail_element.clear()
        mail_element.send_keys(mail)
        mail_element.send_keys(Keys.RETURN)

        pw_element = cls.driver.find_element(By.NAME, 'password')
        pw_element.clear()
        pw_element.send_keys(password)
        pw_element.send_keys(Keys.RETURN)
        time.sleep(1)
        pw_element.send_keys(Keys.RETURN)

        WebDriverWait(cls.driver, 20).until(expected_conditions.url_matches(cls._test_settings.me_page_url))
        time.sleep(4)
        cls._is_logged_in = True
